/*
 * Boss1.java
 *
 * Last modified: <2008/08/22 15:20:34 +0900> By SUMIDA
 */
package game;

import static game.SystemInfo.*;
import static game.Ship.*;
import static game.Shot.*;
import static game.MyShip.*;
import static game.Enemy.*;
import static game.Real.*;
import static game.Logging.*;

public class Boss1 implements EnemyModule {

    public static final short SHIP_FLAG_MOVE_MASK   = 0x0300;
    public static final short SHIP_FLAG_MOVE_LEFT   = 0x0100;
    public static final short SHIP_FLAG_MOVE_RIGHT  = 0x0200;

    public static final short SHIP_ACT_MASK         = 0x00FF;
    public static final short SHIP_ACT_MOVE         = 0x0001;
    public static final short SHIP_ACT_RUSH         = 0x0002;
    public static final short SHIP_ACT_BACK         = 0x0004;
    public static final short SHIP_ACT_CHARGE       = 0x0008;
    public static final short SHIP_ACT_DISCHARGE    = 0x0010;

    /* NOTE : ボスの行動

        SHIP_STAT_APPEAR = 登場

        SHIP_ACT_MOVE|
        SHIP_FLAG_MOVE_LEFT/SHIP_FLAG_MOVE_RIGHT
        SHIP_STAT_MOVE   = 左右移動、5way弾発射
        SHIP_STAT_NORMAL = 待機
        SHIP_STAT_FIRE = レーザーショット発射
        SHIP_ACT_RUSH
        SHIP_STAT_FIRE = 後退後突進
        SHIP_ACT_BACK
        SHIP_STAT_NORMAL = 後退待機

        SHIP_ACT_CHARGE
        SHIP_STAT_MOVE   = 自機正面移動
        SHIP_STAT_NORMAL = 待機
        SHIP_ACT_DISCHARGE
        SHIP_STAT_FIRE   = レーザー発射
        SHIP_STAT_NORMAL = 待機

        ※以後、繰り返し。

        最初の左右移動は、スコアの２桁目の値が偶数の時、左移動から開始する。
        ２回目以降はランダム。
        左右移動はランダムで複数回実行。
        左右の入れ替えは、画面端に到達した時。
     */

    public void init(Ship.Info info) {

        trace(String.format("info=%s, id=%d", info, info.id));
        SystemInfo.status |= SYSTEM_STAT_BOSS1;
        info.flag |= SHIP_FLAG_NODAMAGE|SHIP_FLAG_HEAVY|SHIP_FLAG_NOCLIP;
        info.count = 90;
        info.sprite.options |= 20;
        info.sprite.dy = IntToReal(2)>>1;
    }

    public void action(Ship.Info info) {

        if (info.sprite.dy < 0 && info.sprite.y < IntToReal(-HEIGHT)) {
            /* 下がり過ぎた時は制限をかける */
            info.sprite.y = IntToReal(-HEIGHT);
        }

        switch (info.stat) {
        case SHIP_STAT_APPEAR:
            //trace("SHIP_STAT_APPEAR");
            if (info.count < 0) {
                info.flag &= ~(SHIP_FLAG_NODAMAGE|SHIP_FLAG_USERS);
                info.stat = SHIP_STAT_MOVE;
                info.count = 60;
                info.sprite.dy = 0;

                /* スコアの2桁目によって最初の方向を決める */
                if (((SystemInfo.score / 10 % 10) & 0x01) != 0) {
                    info.sprite.dx = IntToReal(2);
                    info.flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_RIGHT;
                } else {
                    info.sprite.dx = -IntToReal(2);
                    info.flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_LEFT;
                }
            }
            break;

        case SHIP_STAT_MOVE:
            //trace("SHIP_STAT_MOVE");
            if (info.count < 0) {
                /* 時間が来たら次のアクション */
                info.stat = SHIP_STAT_NORMAL;
                info.sprite.dx = 0;
                info.count = 60 * 2;
            }

            if ((info.flag & SHIP_ACT_MOVE) != 0) {

                if (info.sprite.dx < 0 && info.sprite.x < 0) {
                    /* 画面端で反転 */
                    info.flag &= ~SHIP_FLAG_MOVE_MASK;
                    info.flag |= SHIP_FLAG_MOVE_LEFT;
                    info.sprite.dx = -info.sprite.dx;

                } else {
                    Surface.Info surface_info = Surface.getInfo(info.sprite.surface_id);

                    if (info.sprite.dx > 0 && info.sprite.x + IntToReal(surface_info.frame_width) > IntToReal(WIDTH)) {
                        /* 画面端で反転 */
                        info.flag &= ~SHIP_FLAG_MOVE_MASK;
                        info.flag |= SHIP_FLAG_MOVE_RIGHT;
                        info.sprite.dx = -info.sprite.dx;
                    }
                }

            } else {

                Surface.Info surface_info = Surface.getInfo(info.sprite.surface_id);
                Surface.Info surface_ship = Surface.getInfo(ship.sprite.surface_id);

                int x0 = info.sprite.x + (IntToReal(surface_info.frame_width)>>1);
                int x1 = ship.sprite.x + (IntToReal(surface_ship.frame_width)>>1);
                int dx = x1 - x0;

                if (-8 > dx || dx > 8) {
                    /* 自機側に方向変更 */
                    info.sprite.dx = -info.sprite.dx;
                } else {
                    /* 次のアクション */
                    info.sprite.dx = 0;
                    info.count = 60 * 1;
                    info.stat = SHIP_STAT_NORMAL;
                }
            }

            /* 5way弾発射 */
            if (Random(0, 256) < SystemInfo.level) {

                /* 自機方向を算出 */
                double angle = Sprite.getAngle(info.sprite, ship.sprite);
                Shot.Info shot1 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                Shot.Info shot2 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                Shot.Info shot3 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                Shot.Info shot4 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                Shot.Info shot5 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);

                if (shot1 != null) {
                    Shot.fire(shot1, info, 0, 0);
                    Sprite.setAngle(shot1.sprite, angle, IntToReal(2));
                }
                if (shot2 != null) {
                    Shot.fire(shot2, info, 0, 0);
                    Sprite.setAngle(shot2.sprite, angle - 15, IntToReal(2));
                }
                if (shot3 != null) {
                    Shot.fire(shot3, info, 0, 0);
                    Sprite.setAngle(shot3.sprite, angle + 15, IntToReal(2));
                }
                if (shot4 != null) {
                    Shot.fire(shot4, info, 0, 0);
                    Sprite.setAngle(shot4.sprite, angle - 30, IntToReal(2));
                }
                if (shot5 != null) {
                    Shot.fire(shot5, info, 0, 0);
                    Sprite.setAngle(shot5.sprite, angle + 30, IntToReal(2));
                }
            }
            break;

        case SHIP_STAT_NORMAL:
            //trace("SHIP_STAT_NORMAL");
            if (info.count < 0) {
                info.sprite.dx = 0;
                info.count = 5;

                if ((info.flag & (SHIP_ACT_BACK|SHIP_ACT_DISCHARGE)) != 0) {

                    if (info.sprite.y < IntToReal(-16)) {

                        int rnd = (int)Random(-3, 3);

                        info.flag &= ~SHIP_FLAG_USERS;
                        info.stat = SHIP_STAT_MOVE;
                        info.sprite.dy = 0;

                        if (rnd < 0) {
                            info.flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_LEFT;
                            info.sprite.dx = -IntToReal(2);
                            info.count = 120;

                        } else if (rnd > 0) {
                            info.flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_RIGHT;
                            info.sprite.dx = IntToReal(2);
                            info.count = 120;

                        } else {
                            info.flag |= SHIP_ACT_CHARGE;
                        }
                    }

                } else if ((info.flag & SHIP_ACT_CHARGE) != 0) {

                    info.stat = SHIP_STAT_FIRE;
                    info.flag &= ~SHIP_ACT_MASK;
                    info.flag |= SHIP_ACT_DISCHARGE;
                    info.sprite.dy = -IntToReal(1)>>1;

                } else {

                    info.stat = SHIP_STAT_FIRE;

                    /* レーザーショットと突進の選択 */
                    if (SystemInfo.level > 7 && Random(0, 4) < 1) {
                        info.flag &= ~SHIP_ACT_MASK;
                        info.flag |= SHIP_ACT_RUSH;
                        info.sprite.dy = -IntToReal(1)>>1;
                    }
                }
            }
            break;

        case SHIP_STAT_FIRE:
            //trace("SHIP_STAT_FIRE");
            if (info.count < 0) {

                info.count = 5;

                if ((info.flag & SHIP_ACT_RUSH) != 0) {

                    if (info.sprite.dy < 0 && info.sprite.y < -IntToReal(64)) {
                        info.sprite.dx = 0;
                        info.sprite.dy = IntToReal(4);

                    } else if (info.sprite.dy > 0 && info.sprite.y > IntToReal(HEIGHT - 16)) {
                        info.flag &= ~SHIP_ACT_MASK;
                        info.flag |= SHIP_ACT_BACK;
                        info.stat = SHIP_STAT_NORMAL;
                        info.count = 60 * 1;
                        info.sprite.dx = 0;
                        info.sprite.dy = -IntToReal(1);
                    }

                } else if ((info.flag & SHIP_ACT_DISCHARGE) != 0) {

                    /* レーザーの発射 */
                    Shot.Info shot = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY2);

                    if (shot != null) {
                        Shot.fire(shot, info, 0, IntToReal(32));
                        shot.sprite.dy = IntToReal(3);
                    }

                    if (Random(-2, 18) > SystemInfo.level) {
                        info.stat = SHIP_STAT_NORMAL;
                        info.count = 60 * 2;
                        //info.sprite.dy = IntToReal(-1)>>1;
                        info.sprite.dy = 0;
                    }

                } else if (Random(1, 16) < SystemInfo.level) {
                    /* レーザーショットの発射 */
                    Shot.Info shot1 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);
                    Shot.Info shot2 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);
                    Shot.Info shot3 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);
                    Shot.Info shot4 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);

                    if (shot1 != null) {
                        Shot.fire(shot1, info, IntToReal(-34), IntToReal(-20));
                        shot1.sprite.dy = IntToReal(5)>>1;
                    }

                    if (shot2 != null) {
                        Shot.fire(shot2, info, IntToReal(34), IntToReal(-20));
                        shot2.sprite.dy = IntToReal(5)>>1;
                    }

                    if (shot3 != null) {
                        Shot.fire(shot3, info, IntToReal(-18), IntToReal(-4));
                        shot3.sprite.dy = IntToReal(5)>>1;
                    }

                    if (shot4 != null) {
                        Shot.fire(shot4, info, IntToReal(18), IntToReal(-4));
                        shot4.sprite.dy = IntToReal(5)>>1;
                    }

                    if (Random(-4, 18) > SystemInfo.level) {
                        info.stat = SHIP_STAT_MOVE;
                        info.count = 60;
                        info.sprite.dy = 0;

                        /* 逆方向移動 */
                        if ((info.flag & SHIP_FLAG_MOVE_LEFT) != 0) {
                            info.sprite.dx = IntToReal(2);
                            info.flag &= ~SHIP_FLAG_MOVE_MASK;
                            info.flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_RIGHT;
                        } else {
                            info.sprite.dx = -IntToReal(2);
                            info.flag &= ~SHIP_FLAG_MOVE_MASK;
                            info.flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_LEFT;
                        }
                    }
                }
            }
            break;

        default:
            break;
        }
    }

    public void term(Ship.Info info) {

        trace("info=" + info);
        SystemInfo.status &= ~SYSTEM_STAT_BOSS1;
    }
}
