/*
 * Boss2.java
 *
 * Last modified: <2008/08/22 07:03:32 +0900> By SUMIDA
 */
package game;

import static game.SystemInfo.*;
import static game.Ship.*;
import static game.Shot.*;
import static game.MyShip.*;
import static game.Enemy.*;
import static game.Texture.*;
import static game.Real.*;
import static game.Logging.*;

public class Boss2 implements EnemyModule {

    public static final short SHIP_ACT_MASK         = 0x00FF;
    public static final short SHIP_ACT_MOVE         = 0x0001;
    public static final short SHIP_ACT_RUSH         = 0x0002;
    public static final short SHIP_ACT_BACK         = 0x0004;
    public static final short SHIP_ACT_CHARGE       = 0x0008;
    public static final short SHIP_ACT_DISCHARGE    = 0x0010;

    /* NOTE : ボスの行動

        SHIP_STAT_APPEAR = 登場
        SHIP_ACT_MOVE
        SHIP_STAT_MOVE   = 左右移動、レーザーショット発射
        SHIP_STAT_NORMAL = 待機
        SHIP_STAT_FIRE = 3wayx3発射
        SHIP_ACT_RUSH
        SHIP_STAT_FIRE = 突進、光子魚雷
        SHIP_ACT_BACK
        SHIP_STAT_NORMAL = 後退待機

        SHIP_ACT_CHARGE
        SHIP_STAT_MOVE   = 自機正面移動
        SHIP_STAT_NORMAL = 待機
        SHIP_ACT_DISCHARGE
        SHIP_STAT_FIRE   = レーザー、ミサイル、光子魚雷発射
        SHIP_STAT_NORMAL = 待機

        ※以後、繰り返し。
     */

    public void init(Ship.Info info) {

        trace(String.format("info=%s, id=%d", info, info.id));
        SystemInfo.status |= SYSTEM_STAT_BOSS2;
        info.flag |= SHIP_FLAG_NODAMAGE|SHIP_FLAG_HEAVY|SHIP_FLAG_NOCLIP;
        info.count = 150;
        info.sprite.options |= 20;
        info.sprite.dy = IntToReal(2)>>1;
    }

    public void action(Ship.Info info) {

        switch (info.stat) {
        case SHIP_STAT_APPEAR:
            //trace("SHIP_STAT_APPEAR");
            if (info.count < 0) {
                info.flag &= ~(SHIP_FLAG_NODAMAGE|SHIP_FLAG_USERS);
                info.stat = SHIP_STAT_MOVE;
                info.count = 60;
                info.sprite.dy = 0;

                info.flag |= SHIP_ACT_MOVE;
                if (((SystemInfo.score / 10 % 10) & 0x01) != 0) {
                    info.sprite.dx = IntToReal(2);
                } else {
                    info.sprite.dx = -IntToReal(2);
                }
            }
            break;

        case SHIP_STAT_MOVE:
            //trace("SHIP_STAT_MOVE");
            if (info.count < 0) {
                /* 時間が来たら次のアクション */
                info.stat = SHIP_STAT_NORMAL;
                info.sprite.dx = 0;
                info.count = 60 * 2;
            }

            if ((info.flag & SHIP_ACT_MOVE) != 0) {
                Surface.Info surface_info = Surface.getInfo(info.sprite.surface_id);

                if (info.sprite.dx < 0 && info.sprite.x < 0
                        || info.sprite.dx > 0 && info.sprite.x + IntToReal(surface_info.frame_width) > IntToReal(WIDTH)) {
                    /* 画面端で反転 */
                    info.sprite.dx = -info.sprite.dx;
                }

            } else {

                Surface.Info surface_info = Surface.getInfo(info.sprite.surface_id);
                Surface.Info surface_ship = Surface.getInfo(ship.sprite.surface_id);

                int x0 = info.sprite.x + (IntToReal(surface_info.frame_width)>>1);
                int x1 = ship.sprite.x + (IntToReal(surface_ship.frame_width)>>1);
                int dx = x1 - x0;

                if (-8 > dx || dx > 8) {
                    /* 自機側に方向変更 */
                    info.sprite.dx = -info.sprite.dx;
                } else {
                    /* 次のアクション */
                    info.sprite.dx = 0;
                    info.count = 60 * 1;
                    info.stat = SHIP_STAT_NORMAL;
                }
            }

            if ((info.count & ((SystemInfo.level > 10)? 0x07: 0x0f)) == 0) {
                /* レーザーショットの発射 */
                Shot.Info shot1 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);
                Shot.Info shot2 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);

                if (shot1 != null) {
                    Shot.fire(shot1, info, IntToReal(-30), IntToReal(40));
                    shot1.sprite.dy = IntToReal(3);
                }

                if (shot2 != null) {
                    Shot.fire(shot2, info, IntToReal(30), IntToReal(40));
                    shot2.sprite.dy = IntToReal(3);
                }
            }
            break;

        case SHIP_STAT_NORMAL:
            //trace("SHIP_STAT_NORMAL");
            if (info.count < 0) {
                info.sprite.dx = 0;
                info.count = 5;

                if ((info.flag & (SHIP_ACT_BACK|SHIP_ACT_DISCHARGE)) != 0) {

                    if (info.sprite.y < IntToReal(8)) {

                        int rnd = (int)Random(-3, 3);

                        info.flag &= ~SHIP_FLAG_USERS;
                        info.stat = SHIP_STAT_MOVE;
                        info.sprite.dy = 0;

                        if (rnd < 0) {
                            info.flag |= SHIP_ACT_MOVE;
                            info.sprite.dx = -IntToReal(2);
                            info.count = 120;

                        } else if (rnd > 0) {
                            info.flag |= SHIP_ACT_MOVE;
                            info.sprite.dx = IntToReal(2);
                            info.count = 120;

                        } else {
                            info.flag |= SHIP_ACT_CHARGE;
                        }
                    }

                } else if ((info.flag & SHIP_ACT_CHARGE) != 0) {

                    info.stat = SHIP_STAT_FIRE;
                    info.flag &= ~SHIP_ACT_MASK;
                    info.flag |= SHIP_ACT_DISCHARGE;
                    info.sprite.dy = -IntToReal(1)>>1;

                } else {

                    info.stat = SHIP_STAT_FIRE;

                    /* 3way x 3と突進の選択 */
                    if (SystemInfo.level > 7 && Random(0, 4) < 2) {
                        info.flag &= ~SHIP_ACT_MASK;
                        info.flag |= SHIP_ACT_RUSH;
                        info.sprite.dy = IntToReal(1)>>1;
                    }
                }
            }
            break;

        case SHIP_STAT_FIRE:
            //trace("SHIP_STAT_FIRE");
            if (info.count < 0) {

                info.count = 5;

                if ((info.flag & SHIP_ACT_RUSH) != 0) {

                    if (Random(-2, 18) > SystemInfo.level) {
                        /* 光子魚雷の発射 */
                        Shot.Info shot = Shot.getInfo(enemy_shot_mgr, SHOT_ID_PHOTON);

                        if (shot != null) {
                            Shot.fire(shot, info, 0, 0);
                            shot.sprite.dy = IntToReal(-8);
                            shot.sprite.dx = IntToReal((int)Random(-8, 8));
                        }
                    }

                    if (info.sprite.dy > 0 && info.sprite.y > IntToReal(HEIGHT - 16)) {
                        info.flag &= ~SHIP_ACT_MASK;
                        info.flag |= SHIP_ACT_BACK;
                        info.stat = SHIP_STAT_NORMAL;
                        info.count = 60 * 1;
                        info.sprite.dx = 0;
                        info.sprite.dy = -IntToReal(1);
                    }

                } else if ((info.flag & SHIP_ACT_DISCHARGE) != 0) {

                    //if (info.count & 0x04)
                    {
                        /* レーザーの発射 */
                        Shot.Info shot = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY2);

                        if (shot != null) {
                            Shot.fire(shot, info, 0, IntToReal(32));
                            shot.sprite.dy = IntToReal(3);
                        }
                    }

                    if (Random(1, 18) < SystemInfo.level) {
                        /* 光子魚雷の発射 */
                        Shot.Info shot = Shot.getInfo(enemy_shot_mgr, SHOT_ID_PHOTON);

                        if (shot != null) {
                            Shot.fire(shot, info, 0, 0);
                            shot.sprite.dy = IntToReal(-8);
                            shot.sprite.dx = IntToReal((int)Random(-8, 8));
                        }
                    }

                    if (Random(-2, 18) < SystemInfo.level) {
                        /* ミサイルの発射 */
                        Surface.Info surface_info = Surface.getInfo(info.sprite.surface_id);
                        Surface.Info surface_ship = Surface.getInfo(TEX_MISSILE);

                        int x0 = info.sprite.x + (IntToReal(surface_info.frame_width) >> 1);
                        int y0 = info.sprite.y + (IntToReal(surface_info.frame_height) >> 1);
                        int x1 = IntToReal(surface_ship.frame_width) >> 1;
                        int y1 = IntToReal(surface_ship.frame_height) >> 1;

                        Enemy.getInfo(SHIP_ID_MISSILE,
                                x0 - IntToReal(30) - x1,
                                y0 + IntToReal(15) - y1);
                        Enemy.getInfo(SHIP_ID_MISSILE,
                                x0 + IntToReal(30) - x1,
                                y0 + IntToReal(15) - y1);
                    }

                    if (Random(-2, 18) > SystemInfo.level) {
                        info.stat = SHIP_STAT_NORMAL;
                        info.count = 60 * 2;
                        //info.sprite.dy = IntToReal(-1)>>1;
                        info.sprite.dy = 0;
                    }

                } else if (Random(0, 128) > SystemInfo.level) {
                    /* 3way x 3弾発射 */

                    /* 自機方向を算出 */
                    double angle = Sprite.getAngle(info.sprite, ship.sprite);
                    Shot.Info shot1 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                    Shot.Info shot2 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                    Shot.Info shot3 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);

                    Shot.Info shot4 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                    Shot.Info shot5 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                    Shot.Info shot6 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);

                    Shot.Info shot7 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                    Shot.Info shot8 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                    Shot.Info shot9 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);

                    if (shot1 != null) {
                        Shot.fire(shot1, info, 0, 0);
                        Sprite.setAngle(shot1.sprite, angle, IntToReal(2));
                    }
                    if (shot2 != null) {
                        Shot.fire(shot2, info, 0, 0);
                        Sprite.setAngle(shot2.sprite, angle - 15, IntToReal(2));
                    }
                    if (shot3 != null) {
                        Shot.fire(shot3, info, 0, 0);
                        Sprite.setAngle(shot3.sprite, angle + 15, IntToReal(2));
                    }

                    if (shot4 != null) {
                        Shot.fire(shot4, info, IntToReal(-25), 0);
                        Sprite.setAngle(shot4.sprite, angle - 30, IntToReal(2));
                    }
                    if (shot5 != null) {
                        Shot.fire(shot5, info, IntToReal(-25), 0);
                        Sprite.setAngle(shot5.sprite, angle - 45, IntToReal(2));
                    }
                    if (shot6 != null) {
                        Shot.fire(shot6, info, IntToReal(-25), 0);
                        Sprite.setAngle(shot6.sprite, angle - 60, IntToReal(2));
                    }

                    if (shot7 != null) {
                        Shot.fire(shot7, info, IntToReal(25), 0);
                        Sprite.setAngle(shot7.sprite, angle + 30, IntToReal(2));
                    }
                    if (shot8 != null) {
                        Shot.fire(shot8, info, IntToReal(25), 0);
                        Sprite.setAngle(shot8.sprite, angle + 45, IntToReal(2));
                    }
                    if (shot9 != null) {
                        Shot.fire(shot9, info, IntToReal(25), 0);
                        Sprite.setAngle(shot9.sprite, angle + 60, IntToReal(2));
                    }

                    if (Random(-4, 18) > SystemInfo.level) {
                        info.stat = SHIP_STAT_MOVE;
                        info.flag |= SHIP_ACT_MOVE;
                        info.count = 60;
                        info.sprite.dy = 0;

                        if (Random(-2, 2) < 0) {
                            info.sprite.dx = IntToReal(-2);
                        } else {
                            info.sprite.dx = IntToReal(2);
                        }
                    }
                }
            }
            break;

        default:
            break;
        }
    }

    public void term(Ship.Info info) {

        trace("info=" + info);
        SystemInfo.status &= ~SYSTEM_STAT_BOSS2;
    }
}
