/*
 * Effect.java
 *
 * Last modified: <2008/08/23 05:37:04 +0900> By SUMIDA
 */
package game;

import java.util.LinkedList;

import static game.SystemInfo.*;
import static game.Sprite.*;
import static game.Texture.*;
import static game.Real.*;

public class Effect {

    public static final byte EFFECT_ID_EXPLOSION = 0;
    public static final byte EFFECT_ID_BOMB      = 1;
    public static final byte EFFECT_ID_FIRE      = 2;
    public static final byte EFFECT_ID_MOON      = 3;
    public static final byte EFFECT_ID_PLANET    = 4;
    public static final byte EFFECT_ID_GRID      = 5;
    public static final byte EFFECT_ID_SEARCH    = 6;
    public static final byte EFFECT_ID_MAX       = 7;

    public static class Info {
        public int id;
        public Sprite.Info sprite = new Sprite.Info();

        public void init() {
            id = 0;
            sprite.init();
        }
    }

    public static class Manager {
        LinkedList<Info> active = new LinkedList<Info>();
        LinkedList<Info> pool   = new LinkedList<Info>();
    }

    public static class Type {
        int tex_id;
        int options;
        int frame_start;
        int frame_end;
        int frame_interval;

        public Type(int tex_id, int options, int frame_start, int frame_end, int frame_interval) {
            this.tex_id = tex_id;
            this.options = options;
            this.frame_start = frame_start;
            this.frame_end = frame_end;
            this.frame_interval = frame_interval;
        }
    }

    private static Type[] types = {
        new Type(TEX_EXPLOSION, SPRITE_FRAME64|SPRITE_USE_ANIME, 0, 14, 1),
        new Type(TEX_BOMB     , SPRITE_FRAME|SPRITE_USE_ANIME  , 0, 15, 1),
        new Type(TEX_FIRE     , SPRITE_USE_LOWER|SPRITE_FRAME|SPRITE_USE_ANIME|SPRITE_USE_MOVE, 0,  9, 2),
        new Type(TEX_MOON     , SPRITE_USE_MOVE|255, 0, 0, 0),
        new Type(TEX_PLANET   , SPRITE_USE_MOVE|254, 0, 0, 0),
        new Type(TEX_GRID     , SPRITE_FRAME64|SPRITE_USE_ANIME, 0, 3, 1),
        new Type(TEX_SEARCH   , SPRITE_FRAME64, 0, 0, 0),
    };
    
    private static Effect.Manager mgr = new Effect.Manager();

    public static void init() {

        int count = EFFECT_MAX;
        if (mgr.active.isEmpty() && mgr.pool.isEmpty()) {
            while (count-- > 0) {
                mgr.pool.add(new Info());
            }

        } else {
            mgr.pool.addAll(mgr.active);
            mgr.active.clear();
        }

        for (Info info: mgr.pool) {
            info.init();
        }
    }

    public static Info getInfo(int id) {

        Info info = mgr.pool.poll();

        if (info != null) {
            Type type = types[id];

            mgr.active.add(info);
            info.id                    = id;
            info.sprite.surface_id     = type.tex_id;
            info.sprite.options        = SPRITE_VISIBLE|type.options;
            info.sprite.frame_start    = type.frame_start;
            info.sprite.frame_end      = type.frame_end;
            info.sprite.frame_interval = type.frame_interval;
        }

        return info;
    }

    public static void dropInfo(Info info) {

        info.init();
        mgr.pool.add(info);
    }

    public static void action() {

        LinkedList<Info> list = mgr.active;

        mgr.active = new LinkedList<Info>();

        while (!list.isEmpty()) {

            Info info = list.poll();

            if ((info.sprite.options & SPRITE_STOP_ANIME) != 0) {
                dropInfo(info);
                continue;
            }

            {
                Surface.Info surface = Surface.getInfo(info.sprite.surface_id);

                int x0 = RealToInt(info.sprite.x);
                int y0 = RealToInt(info.sprite.y);
                int x1 = x0 + surface.frame_width  -1;
                int y1 = y0 + surface.frame_height -1;

                if (x0 > CLIP_X1 || x1 < CLIP_X0 || y0 > CLIP_Y1 || y1 < CLIP_Y0) {
                    dropInfo(info);
                    continue;
                }
            }

            Sprite.action(info.sprite);
            Sprite.put(info.sprite);
            mgr.active.add(info);
        }
    }

    public static void setPosition(Info info, Ship.Info ship, int ox, int oy) {

        Surface.Info surface_effect = Surface.getInfo(info.sprite.surface_id);
        Surface.Info surface_ship = Surface.getInfo(ship.sprite.surface_id);

        info.sprite.x = ship.sprite.x + ox
            + (IntToReal(surface_ship.frame_width - surface_effect.frame_width) >> 1);
        info.sprite.y = ship.sprite.y + oy
            + (IntToReal(surface_ship.frame_height - surface_effect.frame_height) >> 1);
    }
}
