/*
 * Enemy2.java
 *
 * Last modified: <2008/08/22 09:12:11 +0900> By SUMIDA
 */
package game;

import static game.SystemInfo.*;
import static game.Ship.*;
import static game.Shot.*;
import static game.MyShip.*;
import static game.Enemy.*;
import static game.Real.*;
import static game.Logging.*;

public class Enemy2 implements EnemyModule {

    public void init(Ship.Info info) {

        trace(String.format("info=%s, id=%d", info, info.id));
        info.sprite.options |= 11;
        info.sprite.dy = IntToReal(2);
    }

    public void action(Ship.Info info) {

        switch (info.stat) {
        case SHIP_STAT_APPEAR:
            /* 画面端で反転 */
            if (info.sprite.dx < 0 && info.sprite.x < IntToReal(10)
                || info.sprite.dx > 0 && info.sprite.x > IntToReal(WIDTH - 10)) {
                info.sprite.dx = -info.sprite.dx;
            }

            if (info.count < 0) {
                info.count = 60;
                info.stat = SHIP_STAT_MOVE;
            }
            break;

        case SHIP_STAT_MOVE:
            if (info.count < 0) {

                /* 急速反転 */
                Sprite.setAngle(
                    info.sprite,
                    Sprite.getAngle(info.sprite, ship.sprite) - 180,
                    IntToReal(4));

                info.count = 0;
                info.stat = SHIP_STAT_NORMAL;
            }
            break;

        case SHIP_STAT_NORMAL:
            if (Random(0, 128) < SystemInfo.level) {
                info.count = 10;
                info.stat = SHIP_STAT_FIRE;
            }
            break;

        case SHIP_STAT_FIRE:
            if (info.count < 0 && Random(0, 128) < SystemInfo.level) {
                /* ２連弾の発射 */
                Shot.Info shot1 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);
                Shot.Info shot2 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_BALLET);

                if (shot1 != null) {
                    Shot.fire(shot1, info, IntToReal(-6), 0);
                    shot1.sprite.dy = IntToReal(3);
                }

                if (shot2 != null) {
                    Shot.fire(shot2, info, IntToReal(30), 0);
                    shot2.sprite.dy = IntToReal(3);
                }

                info.count = 0;
                info.stat = SHIP_STAT_NORMAL;
            }
            break;

        default:
            return;
        }
    }

    public void term(Ship.Info info) {
        trace("info=" + info);
    }
}
