/*
 * Enemy4.java
 *
 * Last modified: <2008/08/22 09:19:30 +0900> By SUMIDA
 */
package game;

import static game.SystemInfo.*;
import static game.Ship.*;
import static game.Shot.*;
import static game.MyShip.*;
import static game.Enemy.*;
import static game.Audio.*;
import static game.Real.*;
import static game.Logging.*;

public class Enemy4 implements EnemyModule {

    public void init(Ship.Info info) {

        trace(String.format("info=%s, id=%d", info, info.id));
        info.sprite.options |= 13;
        info.sprite.dy = IntToReal(1);
    }

    public void action(Ship.Info info) {

        switch (info.stat) {
        case SHIP_STAT_APPEAR:
            if (info.count < 0) {
                info.count = 5;
                info.stat = SHIP_STAT_MOVE;
            }
            break;

        case SHIP_STAT_MOVE:
            if (info.count < 0) {
                /* 自機側に方向変更 */
                if (info.sprite.x < ship.sprite.x)
                    info.sprite.dx = IntToReal((int)Random(2, 6));
                else if (info.sprite.x > ship.sprite.x)
                    info.sprite.dx = -IntToReal((int)Random(2, 6));
                else
                    info.sprite.dx = 0;

                info.count = 5;
                info.stat = SHIP_STAT_NORMAL;
            }
            break;

        case SHIP_STAT_NORMAL:
            if (info.count < 0 && Random(0, 128) < SystemInfo.level) {
                info.sprite.dx = 0;
                info.sprite.dy = IntToReal(1)>>1;
                info.count = 5;
                info.stat = SHIP_STAT_FIRE;
            }
            break;

        case SHIP_STAT_FIRE:
            if (info.count < 0 && Random(0, 128) < SystemInfo.level) {
                /* レーザーの発射 */
                Shot.Info shot1 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);
                Shot.Info shot2 = Shot.getInfo(enemy_shot_mgr, SHOT_ID_RAY1);

                if (shot1 != null) {
                    Shot.fire(shot1, info, IntToReal(-16), 0);
                    shot1.sprite.dy = IntToReal(2);
                }

                if (shot2 != null) {
                    Shot.fire(shot2, info, IntToReal(16), 0);
                    shot2.sprite.dy = IntToReal(2);
                    Sound.playEffect(SE_SHOT0);
                }

                info.sprite.dy = IntToReal(1);
                info.count = 20;
                info.stat = SHIP_STAT_MOVE;
            }
            break;

        default:
            return;
        }
    }

    public void term(Ship.Info info) {
        trace("info=" + info);
    }
}
