/*
 * Fps.java
 *
 * Last modified: <2008/08/19 08:33:05 +0900> By SUMIDA
 */
package game;

import java.text.DecimalFormat;

import static game.SystemInfo.*;

/**
 * FPS算出クラス
 * @see http://javagame.skr.jp/index.php?%C0%B5%B3%CE%A4%CAFPS
 */
public class Fps {

    private static double expectFPS   = DEF_EXPECT_FPS;
    private static long   fpsInterval = DEF_FPS_INTERVAL;
    private static long   fpsPeriod   = DEF_FPS_PERIOD;

    private static long   frameCount = 0L;
    private static int    calcInterval = 0;
    private static long   prevCalcTime = 0L;
    private static double actualFPS = 0.0;

    private static DecimalFormat df = new DecimalFormat("FPS=0.0");

    /**
     * 期待FPS値、測定間隔を設定する。
     * @param fps      期待FPS値
     * @param interval 測定間隔(ns)
     */
    public static void set(double fps, long interval) {
        double s = (double)interval / 1000000000L;
        expectFPS = fps;
        fpsInterval = interval;
        fpsPeriod = (long)(s / fps * interval);
    }

    /**
     * 測定を開始する。
     * @param 開始時の時間(ns)
     */
    public static void start(long value) {
        prevCalcTime = value;
    }

    /**
     * 期待FPS値を取得する。
     * @return 期待FPS値
     */
    public static double getExpect() {
        return expectFPS;
    }

    /**
     * 測定間隔を取得する。
     * @return 測定間隔
     */
    public static long getInterval() {
        return fpsInterval;
    }

    /**
     * １フレームあたりの時間
     * @return １フレーム時間
     */
    public static long getPeriod() {
        return fpsPeriod;
    }

    /**
     * FPSを計算する。
     * @return 現在のFPS値
     */
    public static double calc() {
        frameCount++;
        calcInterval += fpsPeriod;

        // 1秒おきにFPSを再計算する
        if (calcInterval >= fpsInterval) {
            // 現在時刻を取得
            long timeNow = System.nanoTime();
            // 実際の経過時間を測定
            long realElapsedTime = timeNow - prevCalcTime;

            // FPSを計算
            // reaEelapsedTimeの単位はnsなのでsに変換する
            actualFPS = ((double) frameCount / realElapsedTime) * 1000000000L;
            System.out.println(df.format(actualFPS));

            frameCount = 0L;
            calcInterval = 0;
            prevCalcTime = timeNow;
        }

        return actualFPS;
    }

    /**
     * FPSを取得する。
     * @return 現在のFPS値
     */
    public static double getActual() {
        return actualFPS;
    }
}
