/*
 * Input.java
 *
 * Last modified: <2008/08/19 23:15:55 +0900> By SUMIDA
 */
package game;

import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

import static java.lang.System.*;
import static java.util.Arrays.*;

public class Input implements KeyListener {

    public static final int PLAYER_MAX = 2;

    public static final int P1 = 0;
    public static final int P2 = 1;

    /* キー押下状態 */
    public static final int INPUT_PRESS_MASK     = 0x000000ff;
    public static final int INPUT_PRESS_START    = 0x00000001;
    public static final int INPUT_PRESS_SELECT   = 0x00000002;
    public static final int INPUT_PRESS_BUTTON1  = 0x00000004;
    public static final int INPUT_PRESS_BUTTON2  = 0x00000008;
    public static final int INPUT_PRESS_UP       = 0x00000010;
    public static final int INPUT_PRESS_DOWN     = 0x00000020;
    public static final int INPUT_PRESS_LEFT     = 0x00000040;
    public static final int INPUT_PRESS_RIGHT    = 0x00000080;

    /* キー押下エッジ */
    public static final int INPUT_KEYDOWN_MASK   = 0x0000ff00;
    public static final int INPUT_KEYDOWN_START  = 0x00000100;
    public static final int INPUT_KEYDOWN_SELECT = 0x00000200;
    public static final int INPUT_KEYDOWN_BUTTON1= 0x00000400;
    public static final int INPUT_KEYDOWN_BUTTON2= 0x00000800;
    public static final int INPUT_KEYDOWN_UP     = 0x00001000;
    public static final int INPUT_KEYDOWN_DOWN   = 0x00002000;
    public static final int INPUT_KEYDOWN_LEFT   = 0x00004000;
    public static final int INPUT_KEYDOWN_RIGHT  = 0x00008000;

    /* キー押上エッジ */
    public static final int INPUT_KEYUP_MASK     = 0x00ff0000;
    public static final int INPUT_KEYUP_START    = 0x00010000;
    public static final int INPUT_KEYUP_SELECT   = 0x00020000;
    public static final int INPUT_KEYUP_BUTTON1  = 0x00040000;
    public static final int INPUT_KEYUP_BUTTON2  = 0x00080000;
    public static final int INPUT_KEYUP_UP       = 0x00100000;
    public static final int INPUT_KEYUP_DOWN     = 0x00200000;
    public static final int INPUT_KEYUP_LEFT     = 0x00400000;
    public static final int INPUT_KEYUP_RIGHT    = 0x00800000;

    /* リピートキー状態 */
    public static final int INPUT_REPEAT_MASK    = 0xff000000;
    public static final int INPUT_REPEAT_START   = 0x01000000;
    public static final int INPUT_REPEAT_SELECT  = 0x02000000;
    public static final int INPUT_REPEAT_BUTTON1 = 0x04000000;
    public static final int INPUT_REPEAT_BUTTON2 = 0x08000000;
    public static final int INPUT_REPEAT_UP      = 0x10000000;
    public static final int INPUT_REPEAT_DOWN    = 0x20000000;
    public static final int INPUT_REPEAT_LEFT    = 0x40000000;
    public static final int INPUT_REPEAT_RIGHT   = 0x80000000;

    public static int[] input = new int[PLAYER_MAX];

    private static int count = 0;
    private static int[] curr_input = new int[PLAYER_MAX];

    public static Input instance = new Input();
    public static boolean quit = false;

    /**
     * コンストラクタ。
     */
    private Input() {
        init();
    }

    public static void init() {
        out.print("input_init ... ");
        quit = false;
        fill(input, 0);
        fill(curr_input, 0);
        /* TODO : ジョイスティックの初期化 */
        out.print(" done.\n");
    }

    public static void read() {
        ++count;

        for (int i = 0; i < PLAYER_MAX; i++) {
            int prev = input[i] & INPUT_PRESS_MASK;
            int curr = curr_input[i];

            input[i] = 0;

            /* TODO : ジョイスティックの状態を読み取る */

            /* KEY PRESS */
            input[i] = curr;

            /* KEY DOWN */
            input[i] |= (curr & ~prev) << 8;

            /* KEY UP */
            input[i] |= (~curr & prev) << 16;

            /* KEY REPEAT */
            if ((count & 0x01) != 0) input[i] |= curr << 24;
        }
    }

    public static void free() {

        out.print("input_free ... ");
        /* TODO : ジョイスティックの解放 */
        out.print("done.\n");
    }

    /**
     * キーがタイプされたとき呼ばれる。
     * 文字入力を検知したい場合はこっちを使う。
     */
    public void keyTyped(KeyEvent e) {}

    /**
     * キーが押されたとき呼ばれる。 
     */
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        switch (key) {
        case KeyEvent.VK_UP:
            curr_input[P1] |= INPUT_PRESS_UP;            /* [↑], [K] */
            break;
        case KeyEvent.VK_DOWN:
            curr_input[P1] |= INPUT_PRESS_DOWN;          /* [↓], [J] */
            break;
        case KeyEvent.VK_LEFT:
            curr_input[P1] |= INPUT_PRESS_LEFT;          /* [←], [H] */
            break;
        case KeyEvent.VK_RIGHT:
            curr_input[P1] |= INPUT_PRESS_RIGHT;         /* [→], [L] */
            break;
        case KeyEvent.VK_SHIFT:
        case KeyEvent.VK_Z:
            curr_input[P1] |= INPUT_PRESS_BUTTON1;       /* 左Shift, [Z] */
            break;
        case KeyEvent.VK_CONTROL:
        case KeyEvent.VK_X:
            curr_input[P1] |= INPUT_PRESS_BUTTON2;       /* 左Ctrl, [X] */
            break;
        case KeyEvent.VK_A:
            curr_input[P1] |= INPUT_PRESS_SELECT;        /* [A] */
            break;
        case KeyEvent.VK_S:
            curr_input[P1] |= INPUT_PRESS_START;         /* [S] */
            break;
        case KeyEvent.VK_ESCAPE:
            quit = true;
            break;
        }
    }

    /**
     * キーが離されたとき呼ばれる。
     */
    public void keyReleased(KeyEvent e) {
        int key = e.getKeyCode();
        switch (key) {
        case KeyEvent.VK_UP:
            curr_input[P1] &= ~INPUT_PRESS_UP;            /* [↑], [K] */
            break;
        case KeyEvent.VK_DOWN:
            curr_input[P1] &= ~INPUT_PRESS_DOWN;          /* [↓], [J] */
            break;
        case KeyEvent.VK_LEFT:
            curr_input[P1] &= ~INPUT_PRESS_LEFT;          /* [←], [H] */
            break;
        case KeyEvent.VK_RIGHT:
            curr_input[P1] &= ~INPUT_PRESS_RIGHT;         /* [→], [L] */
            break;
        case KeyEvent.VK_Z:
            curr_input[P1] &= ~INPUT_PRESS_BUTTON1;       /* 左Shift, [Z] */
            break;
        case KeyEvent.VK_X:
            curr_input[P1] &= ~INPUT_PRESS_BUTTON2;       /* 左Ctrl, [X] */
            break;
        case KeyEvent.VK_A:
            curr_input[P1] &= ~INPUT_PRESS_SELECT;        /* [A] */
            break;
        case KeyEvent.VK_S:
            curr_input[P1] &= ~INPUT_PRESS_START;         /* [S] */
            break;
        }
    }
}
