/*
 * Logging.java
 *
 * Last modified: <2008/08/22 06:57:54 +0900> By SUMIDA
 */
package game;

import static java.lang.System.*;
import static game.SystemInfo.*;

import java.util.Calendar;

/**
 * ログ出力を管理するクラス。
 * 本来であれば、java.util.loggingのAPIに委譲すべきだが…
 * 制限が厳しいので自前かつcommons logging似のメソッドを用意した。
 */
public class Logging {

    public static boolean isTraceEnable() {
        return ((options & SYSTEM_OPT_VERBOSE) != 0) ? true: false;
    }

    public static void info(String str) {
        Calendar now = Calendar.getInstance();
        StackTraceElement e = new Exception().getStackTrace()[1];
        String header = String.format(
            "%4$tY/%4$tm/%4$td %4$tH:%4$tm:%4$tS.%4$tL - %1$s.%2$s(%3$d) [INFO ] ",
            e.getClassName(),
            e.getMethodName(),
            e.getLineNumber(),
            now);
        out.println(header + str);
    }

    public static void warn(String str) {
        if (isTraceEnable()) {
            Calendar now = Calendar.getInstance();
            StackTraceElement e = new Exception().getStackTrace()[1];
            String header = String.format(
                "%4$tY/%4$tm/%4$td %4$tH:%4$tm:%4$tS.%4$tL - %1$s.%2$s(%3$d) [WARN ] ",
                e.getClassName(),
                e.getMethodName(),
                e.getLineNumber(),
                now);
            out.println(header + str);
        }
    }

    public static void trace(String str) {
        if (isTraceEnable()) {
            Calendar now = Calendar.getInstance();
            StackTraceElement e = new Exception().getStackTrace()[1];
            String header = String.format(
                "%4$tY/%4$tm/%4$td %4$tH:%4$tm:%4$tS.%4$tL - %1$s.%2$s(%3$d) [TRACE] ",
                e.getClassName(),
                e.getMethodName(),
                e.getLineNumber(),
                now);
            out.println(header + str);
        }
    }

    public static void error(String str) {
        Calendar now = Calendar.getInstance();
        StackTraceElement e = new Exception().getStackTrace()[1];
        String header = String.format(
            "%4$tY/%4$tm/%4$td %4$tH:%4$tm:%4$tS.%4$tL - %1$s.%2$s(%3$d) [ERROR] ",
            e.getClassName(),
            e.getMethodName(),
            e.getLineNumber(),
            now);
        err.println(header + str);
    }

    public static void error(String str, Throwable th) {
        Calendar now = Calendar.getInstance();
        StackTraceElement e = new Exception().getStackTrace()[1];
        String header = String.format(
            "%4$tY/%4$tm/%4$td %4$tH:%4$tm:%4$tS.%4$tL - %1$s.%2$s(%3$d) [ERROR] ",
            e.getClassName(),
            e.getMethodName(),
            e.getLineNumber(),
            now);
        err.println(header + str);
        err.println(th.getMessage());
        th.printStackTrace();
        //StackTraceElement[] array = th.getStackTrace();
        //for (StackTraceElement element: array) {
        //    out.println(element.toString());
        //}
    }
}
