/*
 * Map.java
 *
 * Last modified: <2008/08/21 18:00:05 +0900> By SUMIDA
 */
package game;

import static game.SystemInfo.*;
import static game.Real.*;
import static game.Ship.*;
import static game.Effect.*;
import static game.Audio.*;
//import static game.Input.*;

public class Map {
    public static final short STAT_READY    = 0x4000;
    public static final short STAT_EMPTY    = 0x1000;

    public static final short STAT_TASKS    = 0x0FFF;
    public static final short TASK_BGM1     = 0x0001;
    public static final short TASK_BGM2     = 0x0002;
    public static final short TASK_MOON     = 0x0003;
    public static final short TASK_PLANET   = 0x0004;

    public static class Info {

        short id;       /* イベントID */
        short count;    /* 出現数     */
        short interval; /* 出現間隔   */
        short status;   /* マップ状態 */

        public Info() {
        }

        public Info(int id, int count, int interval, int status) {
            this.id       = (short)id;
            this.count    = (short)count;
            this.interval = (short)interval;
            this.status   = (short)status;
        }

        public void copy(Info src) {
            this.id       = src.id;
            this.count    = src.count;
            this.interval = src.interval;
            this.status   = src.status;
        }
    }

    private static Info[] map = {
        new Info(  -1, 1,    0, STAT_EMPTY|TASK_BGM1),
        new Info(  -1, 1,    0, STAT_EMPTY|TASK_MOON),
        new Info(  -1, 1, 60*2, STAT_EMPTY),
        new Info(SHIP_ID_ENEMY1, 10, 20, 0),
        new Info(SHIP_ID_ENEMY2, 10, 20, 0),
        new Info(SHIP_ID_ENEMY3, 10, 20, 0),
        new Info(SHIP_ID_ENEMY1, 10, 20, 0),
        new Info(SHIP_ID_ENEMY2, 10, 20, 0),
        new Info(SHIP_ID_ENEMY3, 10, 20, 0),
        new Info(SHIP_ID_ENEMY5, 10, 20, 0),
        new Info(SHIP_ID_ENEMY2, 10, 20, 0),
        new Info(SHIP_ID_ENEMY3, 10, 20, 0),
        new Info(  -1, 1,   60, STAT_EMPTY|TASK_BGM2),
        new Info(SHIP_ID_BOSS1 ,  1, 20, 0),
        new Info(  -1, 1,    0, STAT_EMPTY|TASK_BGM1),
        new Info(  -1, 1, 60*2, STAT_EMPTY),
        new Info(SHIP_ID_ENEMY5, 10, 20, 0),
        new Info(SHIP_ID_ENEMY4, 10, 20, 0),
        new Info(SHIP_ID_ENEMY3, 10, 20, 0),
        new Info(SHIP_ID_ENEMY1, 10, 20, 0),
        new Info(  -1, 1,    0, STAT_EMPTY|TASK_PLANET),
        new Info(SHIP_ID_ENEMY2, 10, 20, 0),
        new Info(SHIP_ID_ENEMY4, 10, 20, 0),
        new Info(SHIP_ID_ENEMY3, 10, 20, 0),
        new Info(SHIP_ID_ENEMY1, 10, 20, 0),
        new Info(SHIP_ID_ENEMY5, 10, 20, 0),
        new Info(  -1, 1,   60, STAT_EMPTY|TASK_BGM2),
        new Info(SHIP_ID_BOSS2 ,  1, 20, 0),
    };

    private static Info curr_map = new Info();
    private static Info empty_map = new Info(0, 0, 0, 0);

    public static void init() {
        SystemInfo.level = ((SystemInfo.options & SYSTEM_OPT_LEVEL) != 0) ?
            SystemInfo.options & SYSTEM_OPT_LEVEL_TYPE: SYSTEM_OPT_NORNAL;
        SystemInfo.targets   = 0;
        SystemInfo.encounter = 0;
        SystemInfo.map_index = 0;
        curr_map.copy(empty_map);
    }

    public static void action() {

        if ((SystemInfo.status & SYSTEM_STAT_BOSS) != 0) return;

        if (SystemInfo.encounter > 0) {
            --SystemInfo.encounter;
            return;
        }

        if ((curr_map.status & STAT_READY) == 0) {
            curr_map.copy(map[SystemInfo.map_index]);
            curr_map.status |= STAT_READY;
        }
/*
        if ((input[P1] & INPUT_KEYUP_SELECT) != 0) {
            Ship.Info enemy = Enemy.getInfo(SHIP_ID_BOSS1, 0, 0);
            //Ship.Info enemy = Enemy.getInfo(SHIP_ID_ENEMY1, 0, 0);
            //Ship.Info enemy = Enemy.getInfo((int)Random(SHIP_ID_ENEMY1, SHIP_ID_MAX), 0, 0);

            if (enemy != null) {
                Surface.Info surface = Surface.getInfo(enemy.sprite.surface_id);

                enemy.sprite.x = IntToReal((int)Random(0, WIDTH - surface.frame_width));
                enemy.sprite.y = IntToReal(-surface.frame_height);
                enemy.sprite.dx = IntToReal((int)Random(-2, 2));
            }
        }
*/
        if (--curr_map.interval < 0) {
            if ((curr_map.status & STAT_EMPTY) == 0) {
                /* 敵機の生成 */
                Ship.Info enemy = Enemy.getInfo(curr_map.id, 0, 0);

                if (enemy != null) {
                    Surface.Info surface = Surface.getInfo(enemy.sprite.surface_id);

                    enemy.sprite.x = IntToReal((int)Random(0, WIDTH / surface.frame_width) * surface.frame_width);
                    enemy.sprite.y = IntToReal(-surface.frame_height);
                    enemy.sprite.dx = IntToReal((int)Random(-2, 2));
                }

            } else {
                int task = curr_map.status & STAT_TASKS;

                switch (task) {
                case TASK_BGM1:
                    action_bgm1();
                    break;

                case TASK_BGM2:
                    action_bgm2();
                    break;

                case TASK_MOON:
                    action_moon();
                    break;

                case TASK_PLANET:
                    action_planet();
                    break;
                }

                curr_map.status &= ~STAT_TASKS;
            }

            if (--curr_map.count > 0) {
                curr_map.interval = map[SystemInfo.map_index].interval;
            } else {
                SystemInfo.encounter = 60;
                curr_map.status &= ~STAT_READY;
                if (++SystemInfo.map_index >= map.length) {
                    SystemInfo.map_index = 0;
                    if (++SystemInfo.level > LEVEL_MAX) SystemInfo.level = LEVEL_MAX;
                }
            }
        }
    }

    private static void action_bgm1() {

        Sound.playMusic(BGM_01, -1);
    }

    private static void action_bgm2() {

        Sound.playMusic(BGM_02, -1);
    }

    private static void action_moon() {

        Effect.Info effect = Effect.getInfo(EFFECT_ID_MOON);

        if (effect != null) {
            Surface.Info surface = Surface.getInfo(effect.sprite.surface_id);

            effect.sprite.x = IntToReal((int)Random(0, WIDTH / surface.frame_width) * surface.frame_width);
            effect.sprite.y = IntToReal(-surface.frame_height);
            effect.sprite.dx = 0;
            effect.sprite.dy = 0x1000;
        }
    }

    private static void action_planet() {

        Effect.Info effect = Effect.getInfo(EFFECT_ID_PLANET);

        if (effect != null) {
            Surface.Info surface = Surface.getInfo(effect.sprite.surface_id);

            effect.sprite.x = IntToReal((int)Random(0, WIDTH / surface.frame_width) * surface.frame_width);
            effect.sprite.y = IntToReal(-surface.frame_height);
            effect.sprite.dx = 0;
            effect.sprite.dy = 0x2000;
        }
    }
}
