/*
 * Sound.java
 *
 * Last modified: <2008/08/20 19:00:03 +0900> By SUMIDA
 */
package game;

import game.MidiEngine;
import game.WaveEngine;

import static java.lang.System.*;

/**
 * BGM、効果音を制御するクラス。
 * 
 * MidiEngine.java、WaveEngine.java の改変版を使用してます。
 * http://javagame.skr.jp/
 */
public class Sound {

    private static MidiEngine midiMgr;
    private static WaveEngine waveMgr;

    /* 初期化処理 */
    public static void init() {

        out.print("sound_init ... ");

        midiMgr = new MidiEngine();
        waveMgr = new WaveEngine();

        out.print("done.\n");
    }

    public static void loadMusic(String file, int id) {

        out.print("music_load(" + file + ") ... ");
        midiMgr.load(id, file);
        out.print("done.\n");
    }

    public static void loadEffect(String file, int id) {

        out.print("effect_load(" + file + ") ... ");
        waveMgr.load(id, file);
        out.print("done.\n");
    }

    /* 終了処理 */
    public static void free() {

        out.print("sound_free ... ");

        /* 演奏停止 */
        stopMusic();

        /* 音楽の解放 */
        midiMgr.close();

        /* 効果音の解放 */
        waveMgr.close();

        out.print("done.\n");
    }

    public static void playMusic(int id, int loop) {

        //out.print("sound_music(%s, %d) ... ", id, loop);
        midiMgr.play(id, loop);
        //out.print("done.\n");
    }

    public static void stopMusic() {

        //out.print("sound_stop_music ... ");
        midiMgr.stop();
        //out.print("done.\n");
    }

    public static void playEffect(int id) {

        //out.print("sound_effect(%d) ... ", id);
        waveMgr.play(id);
        //out.print("done.\n");
    }

    public static void stopEffect() {

        //out.print("sound_stop_effect ... ");
        waveMgr.stop();
        //out.print("done.\n");
    }
}
