/*
 * SystemInfo.java
 *
 * Last modified: <2008/08/22 06:17:21 +0900> By SUMIDA
 */
package game;

import static java.lang.Math.*;

public class SystemInfo {

    /* 定数 */
    public static final String TITLE = "Sample2";

    public static final int WIDTH          = 320;
    public static final int HEIGHT         = 240;
    public static final int DEPTH          =  32;
    public static final int DELAY          =  40;
    public static final int SCREEN_BUFFERS =   2;

    public static final int CHANNEL_MAX    =  16;
    public static final int CHUNK_MAX      =  16;

    public static final int SHIP_MAX       =   3;
    public static final int ENEMY_MAX      =  50;
    public static final int SHIP_SHOT_MAX  =  10;
    public static final int ENEMY_SHOT_MAX = 200;
    public static final int EFFECT_MAX     = 200;
    public static final int LEVEL_MAX      =  15;

    public static final int CLIP_X0        = -32;
    public static final int CLIP_Y0        = -32;
    public static final int CLIP_X1        = (WIDTH+32-1);
    public static final int CLIP_Y1        = (HEIGHT+32-1);

    public static final int SYSTEM_OPT_VERBOSE    = 0x10000000;
    public static final int SYSTEM_OPT_FULLSCREEN = 0x20000000;
    public static final int SYSTEM_OPT_LEVEL      = 0x00000010;
    public static final int SYSTEM_OPT_LEVEL_TYPE = 0x0000000F;
    public static final int SYSTEM_OPT_EASY       = 0x00000000;
    public static final int SYSTEM_OPT_NORNAL     = 0x00000002;
    public static final int SYSTEM_OPT_HARD       = 0x00000005;
    public static final int SYSTEM_OPT_SUPER      = 0x00000008;
    public static final int SYSTEM_OPT_HYPER      = 0x0000000B;

    public static final int SYSTEM_MODE_MASK      = 0x00000003;
    public static final int SYSTEM_MODE_READY     = 0x00000001;
    public static final int SYSTEM_MODE_OVER      = 0x00000002;
    public static final int SYSTEM_MODE_PAUSE     = 0x00000003;

    public static final int SYSTEM_STAT_BOSS1     = 0x00010000;
    public static final int SYSTEM_STAT_BOSS2     = 0x00020000;
    public static final int SYSTEM_STAT_BOSS      = (SYSTEM_STAT_BOSS1|SYSTEM_STAT_BOSS2);

    // 期待するFPS（1秒間に描画するフレーム数）
    public static final int DEF_EXPECT_FPS = 60;

    // FPSを計算する間隔（1s = 10^9ns）
    public static final long DEF_FPS_INTERVAL = 1000000000L;

    // 1フレームで使える持ち時間 (ns)
    public static final long DEF_FPS_PERIOD = (long)(1.0 / DEF_EXPECT_FPS * 1000000000L);

    /* システム情報 */
    public static int    options;
    public static long   score;
    public static int    status;
    public static long   frame;

    public static int    level;
    public static int    targets;
    public static int    encounter;
    public static int    map_index;
    
    public static void init() {
        options   = 0;
        score     = 0;
        status    = 0;
        frame     = 0;

        level     = 0;
        targets   = 0;
        encounter = 0;
        map_index = 0;        
    }

    /**
     * lower 以上 upper 以下の乱数を返す。
     * @param lower 乱数下限
     * @param upper 乱数上限
     * @return 乱数値
     */
    public static double Random(double lower, double upper) {
        //return ((double)random() / RAND_MAX) * (upper - lower) + lower;
        return random() * (upper - lower) + lower;
    }

    /**
     * 単位円上の座標から角度を算出する。
     * @param x ｘ座標
     * @param y ｙ座標
     * @return 角度
     */
    public static double angle(double x, double y) {
        double s = sqrt(x * x + y * y);
        double result = (s != 0) ? asin(x / s) / PI * 180: 0;
        if (y < 0) result = 360 - result;
        return result;
    }
}
